/*
 * usbgem.h: General USB to Ethernet MAC driver framework
 * @(#)usbgem.h	1.4 07/04/22
 * (C) Copyright 2003-2006 Masayuki Murayama KHF04453@nifty.ne.jp
 */

#ifndef _USBGEM_H_
#define _USBGEM_H_

#pragma	ident	"@(#)usbgem.h	1.4 07/04/22"

/*
 * Useful macros and typedefs
 */
#define	USBGEM_NAME_LEN	32

#define USBGEM_TX_TIMEOUT		(drv_usectohz(5*1000000))
#define USBGEM_TX_TIMEOUT_INTERVAL	(drv_usectohz(1*1000000))
#define USBGEM_LINK_WATCH_INTERVAL	(drv_usectohz(1*1000000))	/* 1 sec */

/* general return code */
#define	USBGEM_SUCCESS	0
#define	USBGEM_FAILURE	1

/* return code of usbgem_tx_done */
#define	INTR_RESTART_TX	0x80000000

struct usbgem_stats {
	uint32_t	intr;

	uint32_t	crc;
	uint32_t	errrcv;
	uint32_t	overflow;
	uint32_t	frame;
	uint32_t	missed;
	uint32_t	runt;
	uint32_t	frame_too_long;
	uint32_t	norcvbuf;

	uint32_t	collisions;
	uint32_t	first_coll;
	uint32_t	multi_coll;
	uint32_t	excoll;
	uint32_t	nocarrier;
	uint32_t	defer;
	uint32_t	errxmt;
	uint32_t	underflow;
	uint32_t	xmtlatecoll;
};

/* TX buffer management */
struct txbuf {
	mblk_t			*mp;
	size_t			len;
	clock_t			stime;
	usb_bulk_req_t		*req;
};

struct mcast_addr
{
	struct ether_addr	addr;
	uint32_t		hash;
};

#define	USBGEM_MAXMC		64
#define	USBGEM_MCALLOC		(sizeof(struct mcast_addr) * USBGEM_MAXMC)

#define	SLOT(dp, n)	((n) & ((dp)->ugc.usbgc_tx_list_max - 1))

/*
 * mac soft state
 */
struct usbgem_dev {
	dev_info_t		*dip;
	gld_mac_info_t		*macinfo;
	char			name[USBGEM_NAME_LEN];

	/* pointer to usb private data */
	usb_client_dev_data_t	*reg_data;

	/* usb handles */
	usb_pipe_handle_t	default_pipe;
	usb_pipe_handle_t	bulkin_pipe;
	usb_pipe_handle_t	bulkout_pipe;
	usb_pipe_handle_t	intr_pipe;

	/* usb endpoints */
	usb_ep_descr_t		*ep_default;
	usb_ep_descr_t		*ep_bulkin;
	usb_ep_descr_t		*ep_bulkout;
	usb_ep_descr_t		*ep_intr;

	/* usb policies */
	usb_pipe_policy_t	policy_default;
	usb_pipe_policy_t	policy_bulkin;
	usb_pipe_policy_t	policy_bulkout;
	usb_pipe_policy_t	policy_interrupt;

	/* MAC address information */
	struct ether_addr	cur_addr;
	struct ether_addr	dev_addr;

	/* RX state management */
	kmutex_t		rxlock;
	int			rx_line_busy;

	/* RX buffer management */
	int			rx_buf_len;

	/* TX state management */
	kmutex_t		txlock;
	kcondvar_t		drain_cv;
	int			tx_head;
	int			tx_tail;
#ifdef TX_FIFO_CONTROL
	int			tx_curr;
#endif
	clock_t			tx_start_time;
	boolean_t		tx_blocked;

	/* TX buffer resource management */
	struct txbuf		*tx_list;

	/* NIC hardware state */
	int			nic_state;
#define	NIC_DISCONNECTED	0
#define	NIC_INACTIVE		1
#define	NIC_ACTIVE		2
#define	NIC_ERROR		3	/* need to reset the usb device */

	/* robustness: timer and watchdog */
	timeout_id_t		timeout_id;


	/* MII mamagement */
	boolean_t		anadv_1000fdx;
	boolean_t		anadv_1000hdx;
	boolean_t		anadv_100t4;
	boolean_t		anadv_100fdx;
	boolean_t		anadv_100hdx;
	boolean_t		anadv_10fdx;
	boolean_t		anadv_10hdx;
	boolean_t		full_duplex;
	int			speed;
#define		USBGEM_SPD_10	0
#define		USBGEM_SPD_100	1
#define		USBGEM_SPD_1000	2
	int			flow_control;
#define		FLOW_CONTROL_NONE	0
#define		FLOW_CONTROL_SYMMETRIC	1
#define		FLOW_CONTROL_TX_PAUSE	2
#define		FLOW_CONTROL_RX_PAUSE	3

	boolean_t		no_preamble;
	uint32_t		mii_phy_id;
	int			mii_phy_addr;
	int			mii_state;
#define		MII_STATE_UNKNOWN		0
#define		MII_STATE_RESETTING		1
#define		MII_STATE_AUTONEGOTIATING	2
#define		MII_STATE_AN_DONE		3
#define		MII_STATE_MEDIA_SETUP		4
#define		MII_STATE_LINKUP		5
#define		MII_STATE_LINKDOWN		6

	clock_t			mii_last_check;	/* in tick */
	clock_t			mii_timer;	/* in tick */
#define		MII_RESET_TIMEOUT	drv_usectohz(1000*1000)
#define		MII_AN_TIMEOUT		drv_usectohz(5000*1000)
#define		MII_LINKDOWN_TIMEOUT	drv_usectohz(10000*1000)

	boolean_t		mii_fixedmode;
	timeout_id_t		link_watcher_id;
	boolean_t		mii_supress_msg;

	/* multcast list */
	int			mc_count;
	int			mc_count_req;
	struct mcast_addr	*mc_list;
	int			rxmode;
#define		RXMODE_PROMISC		0x01
#define		RXMODE_ALLMULTI_REQ	0x02
#define		RXMODE_MULTI_OVF	0x04
#define		RXMODE_ALLMULTI		(RXMODE_ALLMULTI_REQ | RXMODE_MULTI_OVF)

	/* statistcs */
	struct usbgem_stats		stats;

	/* pointer to local structure */
	void			*private;
	int			priv_size;

	/* configuration */
	struct usbgem_conf {
		/* name */
		char		usbgc_name[USBGEM_NAME_LEN];
		int		usbgc_ppa;

		/* specification on usb */
		int	usbgc_ifnum;	/* interface number */
		int	usbgc_alt;	/* alternate */

		/* specification on tx engine */
		int		usbgc_tx_list_max;
		boolean_t	usbgc_tx_auto_pad;

		/* specification on rx engine */
		int		usbgc_rx_header_len;
		int		usbgc_rx_list_max;

		/* time out parameters */
		clock_t		usbgc_tx_timeout;
		clock_t		usbgc_tx_timeout_interval;

		/* anto negotiation */
		int		usbgc_flow_control;

		/* MII timeout parameters */
		int	usbgc_mii_mode;
#define		USBGEM_MODE_100BASETX	0
#define		USBGEM_MODE_1000BASET	1
#define		USBGEM_MODE_1000BASETX	2

		/* MII timeout parameters */
		clock_t	usbgc_mii_linkdown_timeout;
		clock_t	usbgc_mii_link_watch_interval;
		clock_t	usbgc_mii_reset_timeout;

		clock_t	usbgc_mii_an_watch_interval;
		clock_t	usbgc_mii_an_timeout;
		clock_t	usbgc_mii_an_wait;	
		clock_t	usbgc_mii_an_delay;

		/* MII configuration */
		int	usbgc_mii_addr_min;
		int	usbgc_mii_linkdown_action;
		int	usbgc_mii_linkdown_timeout_action;
#define		MII_ACTION_NONE		0
#define		MII_ACTION_RESET	1
#define		MII_ACTION_RSA		2
		boolean_t	usbgc_mii_dont_reset;	
		boolean_t	usbgc_mii_an_oneshot;

		/* I/O methods */

		/* mac operation */
		int	(*usbgc_attach_chip)(struct usbgem_dev *dp);
		int	(*usbgc_reset_chip)(struct usbgem_dev *dp);
		int	(*usbgc_init_chip)(struct usbgem_dev *dp);
		int	(*usbgc_start_chip)(struct usbgem_dev *dp);
		int	(*usbgc_stop_chip)(struct usbgem_dev *dp);
		uint32_t (*usbgc_multicast_hash)(struct usbgem_dev *dp, uint8_t *);
		int	(*usbgc_set_rx_filter)(struct usbgem_dev *dp);
		int	(*usbgc_set_media)(struct usbgem_dev *dp);
		int	(*usbgc_get_stats)(struct usbgem_dev *dp);
		u_int	(*usbgc_interrupt)(struct usbgem_dev *dp, mblk_t *mp);

		/* packet manupilation */
		mblk_t	*(*usbgc_tx_make_packet)(struct usbgem_dev *dp, mblk_t *mp);
		mblk_t	*(*usbgc_rx_make_packet)(struct usbgem_dev *dp, mblk_t *mp);
		/* mii operations */
		int	(*usbgc_mii_init)(struct usbgem_dev *dp);
		int	(*usbgc_mii_config)(struct usbgem_dev *dp, int *errp);
		void	(*usbgc_mii_sync)(struct usbgem_dev *dp, int *errp);
		uint16_t (*usbgc_mii_read)(struct usbgem_dev *dp, uint_t reg, int *errp);
		void	(*usbgc_mii_write)(struct usbgem_dev *dp, uint_t reg, uint16_t val, int *errp);
		void	(*usbgc_mii_tune_phy)(struct usbgem_dev *dp, int *errp);
	} ugc;

	int	misc_flag;
#define	USBGEM_NOINTR	0x0001
	timeout_id_t	intr_watcher_id;

	/* buffer size */
	uint_t	mtu;

	/* performance tuning parameters */
	uint_t	txthr;		/* tx fifo threshoold */
	uint_t	txmaxdma;	/* tx max dma burst size */ 
	uint_t	rxthr;		/* rx fifo threshoold */
	uint_t	rxmaxdma;	/* tx max dma burst size */ 

#ifdef USBGEM_DEBUG_LEVEL
	int	tx_cnt;
#endif
};

#define	USBGEM_ATTACH_CHIP(dp)	(dp->ugc.usbgc_attach_chip)(dp)
#define	USBGEM_RESET_CHIP(dp)	(dp->ugc.usbgc_reset_chip)(dp)
#define	USBGEM_INIT_CHIP(dp)	(dp->ugc.usbgc_init_chip)(dp)
#define	USBGEM_START_CHIP(dp)	(dp->ugc.usbgc_start_chip)(dp)
#define	USBGEM_STOP_CHIP(dp)	(dp->ugc.usbgc_stop_chip)(dp)
#define	USBGEM_SET_MEDIA(dp)	(dp->ugc.usbgc_set_media)(dp)
#define	USBGEM_MULTICAST_HASH(dp, addr) (dp->ugc.usbgc_multicast_hash)(dp, addr)
#define	USBGEM_SET_RX_FILTER(dp)	(dp->ugc.usbgc_set_rx_filter)(dp)
#define	USBGEM_GET_STATS_CHIP(dp)	(dp->ugc.usbgc_get_stats)(dp)
#define	USBGEM_INTERRUPT(dp, mp)	(dp->ugc.usbgc_interrupt)(dp, mp)

#define	USBGEM_TX_MAKE_PKT(dp, mp)	(dp->ugc.usbgc_tx_make_packet)(dp, mp)
#define	USBGEM_RX_MAKE_PKT(dp, mp)	(dp->ugc.usbgc_rx_make_packet)(dp, mp)


#define	USBGEM_MII_INIT(dp)		(dp->ugc.usbgc_mii_init)(dp)
#define	USBGEM_MII_CONFIG(dp, errp)	(dp->ugc.usbgc_mii_config)(dp, errp)
#define	USBGEM_MII_SYNC(dp, errp)	(dp->ugc.usbgc_mii_sync)(dp, errp)
#define	USBGEM_MII_READ(dp, reg, errp)	(dp->ugc.usbgc_mii_read)(dp, reg, errp)
#define	USBGEM_MII_WRITE(dp, reg, val, errp)	(dp->ugc.usbgc_mii_write)(dp, reg, val, errp)
#define	USBGEM_MII_TUNE_PHY(dp, errp)	(dp->ugc.usbgc_mii_tune_phy)(dp, errp)

/*
 * Exported functions
 */
int usbgem_ctrl_out(struct usbgem_dev *dp,
	uint8_t reqt, uint8_t req, uint16_t val, uint16_t ix, uint16_t len,
	void *bp, int size);

int usbgem_ctrl_in(struct usbgem_dev *dp,
	uint8_t reqt, uint8_t req, uint16_t val, uint16_t ix, uint16_t len,
	void *bp, int size);

int usbgem_ctrl_out_val(struct usbgem_dev *dp,
	uint8_t reqt, uint8_t req, uint16_t val, uint16_t ix, uint16_t len,
	uint32_t v);

int usbgem_ctrl_in_val(struct usbgem_dev *dp,
	uint8_t reqt, uint8_t req, uint16_t val, uint16_t ix, uint16_t len,
	void *valp);

void usbgem_generate_macaddr(struct usbgem_dev *, uint8_t *);
boolean_t usbgem_get_mac_addr_conf(struct usbgem_dev *);
int usbgem_mii_init_default(struct usbgem_dev *);
int usbgem_mii_config_default(struct usbgem_dev *, int *errp);
void usbgem_restart_tx(struct usbgem_dev *);
boolean_t usbgem_tx_done(struct usbgem_dev *, int);
void usbgem_receive(struct usbgem_dev *);
struct usbgem_dev *usbgem_do_attach(dev_info_t *,
		struct usbgem_conf *, void *, int);
int usbgem_do_detach(dev_info_t *);

uint32_t usbgem_ether_crc_le(uint8_t *addr);
uint32_t usbgem_ether_crc_be(uint8_t *addr);

#define	usbgem_rsrv	gld_rsrv
#define	usbgem_open	gld_open
#define	usbgem_close	gld_close
#define	usbgem_wput	gld_wput
#define	usbgem_wsrv	gld_wsrv
#define	usbgem_getinfo	gld_getinfo

#endif /* _USBGEM_H_ */
